from ._utils import *
from ..models import AccessObject, Assignment, Level
from ..forms import AccessFormField
from django.forms import ModelForm, ValidationError
from django.forms.models import modelformset_factory, ModelForm
import datetime


class AssignmentModelForm(ModelForm):
    class Meta:
        model = Assignment
        exclude = ('access_object', )


AssignmentFormSet = modelformset_factory(Assignment, form=AssignmentModelForm)


class AccessFormFieldTestCase(ArmAccessTestCase):
    def setUp(self):
        super(AccessFormFieldTestCase, self).setUp()
        self.field = AccessFormField()

    def testCleanExistingAccessObject(self):
        obj = AccessObject.objects.create()
        self.assertEqual(self.field.clean(obj.id), obj.id)

    def testCleanEmptyFormSet(self):
        formset = AssignmentFormSet({
                'test-TOTAL_FORMS': 0,
                'test-INITIAL_FORMS': 0,
            }, prefix='test')
        obj_id = self.field.clean(formset)
        obj = AccessObject.objects.get(id=obj_id)
        self.assertEqual(0, obj.assignments.count())

    def testCleanPublic(self):
        obj = self.field.clean(None)
        self.assertTrue(obj is None)

    def testCleanInvalidValue(self):
        with self.assertRaises(ValidationError):
            obj = self.field.clean('invalid')

    def testCleanInvalidFormSet(self):
        formset = AssignmentFormSet({
                'test-TOTAL_FORMS': 1,
                'test-INITIAL_FORMS': 0,
                'test-0-id': '',
                'test-0-level': '',
                'test-0-start_date': datetime.datetime.now(),
                'test-0-end_date': datetime.datetime.now(),
            }, prefix='test')
        with self.assertRaises(ValidationError):
            obj = self.field.clean(formset)

    def testCleanValidFormSet(self):
        foo = Level.objects.create(name='foo')
        bar = Level.objects.create(name='bar')
        formset = AssignmentFormSet({
                'test-TOTAL_FORMS': 2,
                'test-INITIAL_FORMS': 0,
                'test-0-id': '',
                'test-0-level': foo.id,
                'test-0-start_date': datetime.datetime.now(),
                'test-0-end_date': datetime.datetime.now(),
                'test-1-id': '',
                'test-1-level': bar.id,
                'test-1-start_date': datetime.datetime.now(),
                'test-1-end_date': datetime.datetime.now(),
            }, prefix='test')
        self.assertTrue(formset.is_valid())
        obj_id = self.field.clean(formset)
        obj = AccessObject.objects.get(id=obj_id)
        self.assertEqual(2, obj.assignments.count())
        assignments = obj.assignments.all()
        levels = [assignment.level for assignment in assignments]
        self.assertTrue(foo in levels)
        self.assertTrue(bar in levels)



import datetime
import StringIO
import sys
import shutil

from south.migration import Migrations

from django.contrib.auth.models import User
from django.core import management

from ._utils import *
from ..models import *
from .arm_access_support.models import *


class AccessFieldTestCase(ArmAccessTestCase):
    def setUp(self):
        super(AccessFieldTestCase, self).setUp()
        self.foo_level = Level.objects.create(name='foo')
        self.bar_level = Level.objects.create(name='bar')
        self.now = datetime.datetime.now()
        self.past = self.now + datetime.timedelta(days=-5)
        self.far_past = self.now + datetime.timedelta(days=-15)
        self.future = self.now + datetime.timedelta(days=-5)

    def testNoInteractionMeansNullAccessObject(self):
        obj = ArmAccessSupportContent.objects.create()
        self.assertEquals(0, AccessObject.objects.count())
        self.assertEquals(None, obj.access)

    def testSettingAccessToNoneExplicitly(self):
        obj = ArmAccessSupportContent.objects.create()
        obj.access = None
        self.assertEquals(0, AccessObject.objects.count())
        self.assertEquals(None, obj.access)

    def testSettingAccessToEmptyList(self):
        obj = ArmAccessSupportContent.objects.create()
        obj.access = []
        self.assertEquals(1, AccessObject.objects.count())
        self.assertEquals(0, Assignment.objects.count())
        self.assertNotEquals(None, obj.access)
        self.assertEquals([], list(obj.access.current_assignments))

    def testSettingAccessToSingleAssignment(self):
        obj = ArmAccessSupportContent.objects.create()
        assign = Assignment(level=self.foo_level, start_date=self.past)
        obj.access = assign
        self.assertEquals(1, AccessObject.objects.count())
        self.assertEquals(1, Assignment.objects.count())
        self.assertNotEquals(None, obj.access)
        self.assertEquals([assign], list(obj.access.current_assignments))

    def testSettingAccessToSingleAssignmentInArray(self):
        obj = ArmAccessSupportContent.objects.create()
        assign = Assignment(level=self.foo_level, start_date=self.past)
        obj.access = [assign]
        self.assertEquals(1, AccessObject.objects.count())
        self.assertEquals(1, Assignment.objects.count())
        self.assertNotEquals(None, obj.access)
        self.assertEquals([assign], list(obj.access.current_assignments))

    def testSettingAccessToMultipleAssignmentInArray(self):
        obj = ArmAccessSupportContent.objects.create()
        assign_foo = Assignment(level=self.foo_level, start_date=self.past)
        assign_bar = Assignment(level=self.bar_level, start_date=self.past)
        obj.access = [assign_foo, assign_bar]
        self.assertEquals(1, AccessObject.objects.count())
        self.assertEquals(2, Assignment.objects.count())
        self.assertNotEquals(None, obj.access)
        self.assertEquals([assign_foo, assign_bar],
                list(obj.access.current_assignments))

    def testSettingAccessToNull(self):
        obj = ArmAccessSupportContent.objects.create()
        assign_foo = Assignment(level=self.foo_level, start_date=self.past)
        assign_bar = Assignment(level=self.bar_level, start_date=self.past)
        obj.access = [assign_foo, assign_bar]
        obj.access = None
        self.assertEquals(0, AccessObject.objects.count())
        self.assertEquals(0, Assignment.objects.count())
        self.assertEquals(None, obj.access)

class AccessFieldSouthTestCase(ArmAccessTestCase):
    def testGenerateSouthMigration(self):
        tmp = StringIO.StringIO()
        sys.stdout = tmp
        sys.stderr = tmp

        management.call_command(
            "schemamigration",
            "arm_access_support",
            "-",
            initial=True,
        )
        migrations = Migrations("arm_access_support")
        shutil.rmtree(migrations.migrations_dir())

        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__



# -*- coding: utf-8 -*-
from lettuce import *

from django.conf import settings
from django.contrib.auth.models import User
import fudge
import random
import sys
import datetime as dt

from armstrong.core.arm_access.tests.arm_access_support.models import Content
from armstrong.core.arm_access.models import *
from armstrong.core.arm_access.paywalls.subscription import (
        SubscriptionPaywall, SubscriptionChecker)


@before.all
def setup_everything():
    world.everyone = Level.objects.create(name="everyone",
                                                is_protected=False)
    world.premium = Level.objects.create(name="premium",
                                               is_protected=True)
    world.user = User.objects.create(username="jjohnson", is_active=True)
    world.user.set_password('1234')


@before.each_scenario
def setup_scenario(scenario):
    pass


@after.each_scenario
def teardown_scenario(scenario):
    Content.objects.all().delete()
    AccessMembership.objects.all().delete()
    Assignment.objects.all().delete()


@step(u'A piece of content exists')
def a_piece_of_content_exists(step):
    world.content = Content.objects.create(name="The content")
    world.content.access = []


@step(u'it has no access defined')
def it_has_no_access_defined(step):
    world.content.access = None


@step(u'a user has no access levels defined')
def a_user_has_no_access_levels_defined(step):
    pass


@step(u'the default has_access backend is configured')
def the_default_has_access_backend_is_configured(step):
    world.backend = SubscriptionPaywall()


@step(u'the access check is performed')
def the_access_check_is_performed(step):
    request = fudge.Fake().has_attr(user=world.user)
    world.result = SubscriptionChecker.has_permission(request, world.content)


@step(u'access should be (.*)')
def access_should_be_denied(step, allowed_or_denied):
    if allowed_or_denied == 'allowed':
        assert world.result == True
    else:
        assert world.result == False


@step(u'it has a (.*) access node with a date in the (.*)')
def it_has_an_access_node_with_a_date(step, node_type, date):
    access_level = Level.objects.get(name=node_type)
    if date == 'future':
        access_date = dt.datetime.now() + dt.timedelta(days=5)
    else:
        access_date = dt.datetime.now() + dt.timedelta(days=-5)
    world.content.access.create(level=access_level, start_date=access_date)


@step(u'a user has the premium access level')
def a_user_has_the_premium_access_level(step):
    kwargs = {}
    kwargs['active'] = True
    kwargs['start_date'] = dt.datetime.now() + dt.timedelta(days=-5)
    kwargs['end_date'] = dt.datetime.now() + dt.timedelta(days=5)
    kwargs['level'] = world.premium
    kwargs['user'] = world.user
    AccessMembership.objects.create(**kwargs)



from ._utils import *
from ..models import AccessObject, Assignment, Level
from ..widgets import *
from django.forms import ModelForm, ValidationError
from django.forms.models import modelformset_factory, ModelForm
import datetime


class AccessWidgetTestCase(ArmAccessTestCase):
    def setUp(self):
        super(AccessWidgetTestCase, self).setUp()
        self.widget = AccessWidget()

    def testParseForPublic(self):
        name = '%i' % random.randint(1000, 10000)
        data = {
                    name + '_is_public': True
                }
        value = self.widget.value_from_datadict(data, [], name)

    def testParseForEmptyAssignments(self):
        name = '%i' % random.randint(1000, 10000)
        data = {
                 name + '-assignments-TOTAL_FORMS': u'0',
                 name + '-assignments-INITIAL_FORMS': u'0',
                 name + '-assignments-MAX_NUM_FORMS': u'',
                }
        value = self.widget.value_from_datadict(data, [], name)
        self.assertEqual(0, len(value))

    def testParseFor3Assignments(self):
        name = '%i' % random.randint(1000, 10000)
        foo = Level.objects.create(name='foo')
        bar = Level.objects.create(name='bar')
        baz = Level.objects.create(name='baz')
        data = {
                 name + '-assignments-TOTAL_FORMS': u'3',
                 name + '-assignments-INITIAL_FORMS': u'0',
                 name + '-assignments-MAX_NUM_FORMS': u'',
                 name + '-assignments-0-id': u'',
                 name + '-assignments-0-level': u'%i' % foo.id,
                 name + '-assignments-0-start_date_0': u'2011-08-12',
                 name + '-assignments-0-start_date_1': u'12:50:56',
                 name + '-assignments-0-end_date_1': u'23:59:59',
                 name + '-assignments-0-end_date_0': u'9999-12-31',
                 name + '-assignments-1-id': u'',
                 name + '-assignments-1-level': u'%i' % baz.id,
                 name + '-assignments-1-start_date_0': u'2011-08-12',
                 name + '-assignments-1-start_date_1': u'13:13:14',
                 name + '-assignments-1-end_date_0': u'',
                 name + '-assignments-1-end_date_1': u'',
                 name + '-assignments-2-id': u'',
                 name + '-assignments-2-level': u'%i' % bar.id,
                 name + '-assignments-2-start_date_0': u'2011-08-12',
                 name + '-assignments-2-start_date_1': u'13:13:14',
                 name + '-assignments-2-end_date_0': u'',
                 name + '-assignments-2-end_date_1': u'',
             }
        value = self.widget.value_from_datadict(data, [], name)
        self.assertEqual(3, len(value))
        self.assertTrue(value.is_valid())

    def testContextWithId(self):
        name = '%i' % random.randint(1000, 10000)
        foo = Level.objects.create(name='foo')
        bar = Level.objects.create(name='bar')
        obj = AccessObject.objects.create()
        obj.create(level=foo, start_date=datetime.datetime.now())
        obj.create(level=bar, start_date=datetime.datetime.now())
        context = self.widget.get_context(name, obj.id)
        self.assertFalse(context['hidden'])
        self.assertEqual(name, context['name'])
        self.assertFalse(context['is_public'])
        self.assertEqual(2, len(context['assignments']))

    def testContextWithNone(self):
        name = '%i' % random.randint(1000, 10000)
        context = self.widget.get_context(name, None)
        self.assertFalse(context['hidden'])
        self.assertEqual(name, context['name'])
        self.assertTrue(context['is_public'])
        # we want a blank field with a default start_date
        self.assertEqual(1, len(context['assignments']))
        now = datetime.datetime.now()
        form = context['assignments'][0]
        self.assertTrue(-20 < (now - form.initial['start_date']).seconds < 20)

    def testContextWithFormset(self):
        name = '%i' % random.randint(1000, 10000)
        foo = Level.objects.create(name='foo')
        bar = Level.objects.create(name='bar')
        baz = Level.objects.create(name='baz')
        data = {
                 name + '-assignments-TOTAL_FORMS': u'3',
                 name + '-assignments-INITIAL_FORMS': u'0',
                 name + '-assignments-MAX_NUM_FORMS': u'',
                 name + '-assignments-0-id': u'',
                 name + '-assignments-0-level': u'%i' % foo.id,
                 name + '-assignments-0-start_date_0': u'2011-08-12',
                 name + '-assignments-0-start_date_1': u'12:50:56',
                 name + '-assignments-0-end_date_1': u'23:59:59',
                 name + '-assignments-0-end_date_0': u'9999-12-31',
                 name + '-assignments-1-id': u'',
                 name + '-assignments-1-level': u'%i' % baz.id,
                 name + '-assignments-1-start_date_0': u'2011-08-12',
                 name + '-assignments-1-start_date_1': u'13:13:14',
                 name + '-assignments-1-end_date_0': u'',
                 name + '-assignments-1-end_date_1': u'',
                 name + '-assignments-2-id': u'',
                 name + '-assignments-2-level': u'%i' % bar.id,
                 name + '-assignments-2-start_date_0': u'',
                 name + '-assignments-2-start_date_1': u'',
                 name + '-assignments-2-end_date_0': u'',
                 name + '-assignments-2-end_date_1': u'',
             }
        value = self.widget.value_from_datadict(data, [], name)
        context = self.widget.get_context(name, value)
        self.assertEqual(name, context['name'])
        self.assertFalse(context['is_public'])
        self.assertEqual(3, len(context['assignments']))



from armstrong.dev.tests.utils import ArmstrongTestCase
from armstrong.dev.tests.utils.concrete import *
from armstrong.dev.tests.utils.users import *

import fudge


class ArmAccessTestCase(ArmstrongTestCase):
    def setUp(self):
        super(ArmAccessTestCase, self).setUp()

    def tearDown(self):
        super(ArmAccessTestCase, self).tearDown()



from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

from .access_memberships import *
from .fields import *
from .subscription import *
from .forms import *
from .widgets import *



from django import forms
from .widgets import AccessWidget
from .models import AccessObject


class AccessFormField(forms.Field):
    widget = AccessWidget

    def __init__(self, *args, **kwargs):
        super(AccessFormField, self).__init__(self, *args, **kwargs)

    def clean(self, value):
        if value is None:
            return None
        if hasattr(value, 'is_valid'):
            if not value.is_valid():
                raise forms.ValidationError("Invalid Access Definition")
            obj = AccessObject.objects.create()
            obj.add(*(value.save(commit=False)))
            return obj.id
        try:
            return int(value)
        except:
            raise forms.ValidationError("Value could not be converted")
